@@ -18,6 +18,7 @@ splitfactor <- function(data, var.name, replace = TRUE, sep = "_", drop.level =
     #data=data set to be changed
     
     if (is.data.frame(data)) {
+        data <- as.data.frame(data)
         if (check) {
             factor.names <- names(data)[sapply(data, function(x) is.factor(x) || is.character(x))]
             if (missing(var.name)) {
@@ -278,6 +279,7 @@ get.w.matchit <- function(m,...) {
     return(m$weights)
 }
 get.w.ps <- function(ps, stop.method = NULL, estimand = NULL, ...) {
+    estimand <- tolower(estimand)
     if (length(stop.method) > 0) {
         if (any(is.character(stop.method))) {
             rule1 <- names(ps$w)[sapply(names(ps$w), function(x) any(startsWith(tolower(x), tolower(stop.method))))]
@@ -318,9 +320,9 @@ get.w.ps <- function(ps, stop.method = NULL, estimand = NULL, ...) {
     w <- setNames(as.data.frame(lapply(seq_along(s), function(p) {
         if (estimand[p] == "att") ps$treat + (1-ps$treat)*ps$ps[,s[p]]/(1-ps$ps[,s[p]])
         else if (estimand[p] == "ate") ps$treat/ps$ps[,s[p]] + (1-ps$treat)/(1-ps$ps[,s[p]])
-        else (1-ps$treat) + ps$treat*ps$ps[,s[p]]/(1-ps$ps[,s[p]])})),
+        else if (estimand[p] == "atc") (1-ps$treat) + ps$treat*ps$ps[,s[p]]/(1-ps$ps[,s[p]])})),
         ifelse(tolower(substr(s, nchar(s)-2, nchar(s))) == tolower(estimand), s, paste0(s, " (", toupper(estimand), ")")))
-    
+    if (ncol(w) == 1) w <- w[[1]]
     return(w)
 }
 get.w.mnps <- function(mnps, stop.method = NULL, ...) {
@@ -369,6 +371,9 @@ get.w.mnps <- function(mnps, stop.method = NULL, ...) {
             w[mnps$treatVar == i, s] <- get.w.ps(mnps$psList[[i]])[mnps$psList[[i]]$treat == TRUE, s]
         }
     }
+    
+    if (ncol(w) == 1) w <- w[[1]]
+    
     return(w)
 }
 get.w.Match <- function(M,  ...) {
@@ -392,7 +397,8 @@ get.w.Match <- function(M,  ...) {
     return(o.data2$weights)
 }
 get.w.CBPS <- function(c, estimand = NULL, ...) {
-    if ("CBPSContinuous" %in% class(c)) { #continuous
+    estimand <- tolower(estimand)
+    if ("CBPSContinuous" %in% class(c) || is.factor(c$y)) { #continuous
         return(c$weights)
     }
     else {
@@ -407,15 +413,21 @@ get.w.CBPS <- function(c, estimand = NULL, ...) {
             else estimand <- "ate"
         }
         
-        estimand <- match.arg(tolower(estimand), c("att", "ate"))
+        estimand <- match.arg(tolower(estimand), c("att", "atc", "ate"))
         if (estimand == "att") {
             return(ifelse(t == 1, 1, ps/(1-ps)))
         }
+        if (estimand == "atc") {
+            return(ifelse(t == 1, (1-ps)/ps, 1))
+        }
         else if (estimand == "ate") {
             return(ifelse(t == 1, 1/ps, 1/(1-ps)))
         }
     }
 }
+get.w.npCBPS <- function(c, estimand = NULL, ...) {
+    return(c$weights)
+}
 get.w.ebalance <- function(e, treat, ...) {
     if (missing(treat)) stop("treat must be specified.", call. = FALSE)
     
@@ -427,9 +439,13 @@ get.w.ebalance <- function(e, treat, ...) {
     weights[treat == 0] <- e$w
     return(weights)
 }
+get.w.ebalance.trim <- get.w.ebalance
 get.w.optmatch <- function(o, treat, ...) {
     return(match.strata2weights(o, treat = treat, covs = NULL))
 }
+get.w.weightit <- function(weightit, ...) {
+    return(weightit$weights)
+}
 
 #For cobalt
 word.list <- function(word.list = NULL, and.or = c("and", "or"), is.are = FALSE, quotes = FALSE) {
@@ -472,6 +488,13 @@ word.list <- function(word.list = NULL, and.or = c("and", "or"), is.are = FALSE,
     }
     return(out)
 }
+expand.grid_string <- function(..., collapse = "") {
+    return(apply(expand.grid(...), 1, paste, collapse = collapse))
+}
+nunique <- function(x) {
+    if (is.factor(x)) return(nlevels(x))
+    else return(length(unique(x)))
+}
 
 #Under construction
 inxnoty <- function(x, y) {
